@@ -317,7 +317,7 @@ gst_visual_src_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstVisual *visual = GST_VISUAL (gst_pad_get_parent (pad));
   GstStructure *structure;
-  gint depth;
+  gint depth, pitch;
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -336,12 +336,12 @@ gst_visual_src_setcaps (GstPad * pad, GstCaps * caps)
   visual_video_set_depth (visual->video,
       visual_video_depth_enum_from_value (depth));
   visual_video_set_dimension (visual->video, visual->width, visual->height);
+  pitch = GST_ROUND_UP_4 (visual->width * visual->video->bpp);
+  visual_video_set_pitch (visual->video, pitch);
   visual_actor_video_negotiate (visual->actor, 0, FALSE, FALSE);
 
   /* precalc some values */
-  visual->outsize =
-      visual->video->height * GST_ROUND_UP_4 (visual->video->width) *
-      visual->video->bpp;
+  visual->outsize = visual->video->height * pitch;
   visual->spf =
       gst_util_uint64_scale_int (visual->rate, visual->fps_d, visual->fps_n);
   visual->duration =
@@ -370,6 +370,8 @@ gst_visual_sink_setcaps (GstPad * pad, GstCaps * caps)
   gst_structure_get_int (structure, "channels", &visual->channels);
   gst_structure_get_int (structure, "rate", &visual->rate);
 
+  /* this is how many samples we need to fill one frame at the requested
+   * framerate. */
   if (visual->fps_n != 0) {
     visual->spf =
         gst_util_uint64_scale_int (visual->rate, visual->fps_d, visual->fps_n);
@@ -385,15 +387,16 @@ gst_vis_src_negotiate (GstVisual * visual)
 {
   GstCaps *othercaps, *target, *intersect;
   GstStructure *structure;
-  const GstCaps *templ;
+  GstCaps *caps;
 
-  templ = gst_pad_get_pad_template_caps (visual->srcpad);
+  caps = gst_pad_get_caps (visual->srcpad);
 
   /* see what the peer can do */
   othercaps = gst_pad_peer_get_caps (visual->srcpad);
   if (othercaps) {
-    intersect = gst_caps_intersect (othercaps, templ);
+    intersect = gst_caps_intersect (othercaps, caps);
     gst_caps_unref (othercaps);
+    gst_caps_unref (caps);
 
     if (gst_caps_is_empty (intersect))
       goto no_format;
@@ -401,9 +404,14 @@ gst_vis_src_negotiate (GstVisual * visual)
     target = gst_caps_copy_nth (intersect, 0);
     gst_caps_unref (intersect);
   } else {
-    target = gst_caps_ref ((GstCaps *) templ);
+    /* need a copy, we'll be modifying it when fixating */
+    target = gst_caps_copy (caps);
+    gst_caps_unref (caps);
   }
 
+  /* fixate in case something is not fixed. This does nothing if the value is
+   * already fixed. For video we always try to fixate to something like
+   * 320x240x30 by convention. */
   structure = gst_caps_get_structure (target, 0);
   gst_structure_fixate_field_nearest_int (structure, "width", 320);
   gst_structure_fixate_field_nearest_int (structure, "height", 240);
@@ -541,11 +549,6 @@ get_buffer (GstVisual * visual, GstBuffer ** outbuf)
   if (ret != GST_FLOW_OK)
     return ret;
 
-  /* this is bad and should not happen. When the alloc function
-   * returns _OK, core ensures we have a valid buffer. */
-  if (*outbuf == NULL)
-    return GST_FLOW_ERROR;
-
   return GST_FLOW_OK;
 }
 
@@ -586,15 +589,23 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
 
   gst_adapter_push (visual->adapter, buffer);
 
-  avail = gst_adapter_available (visual->adapter);
-  GST_DEBUG_OBJECT (visual, "avail now %u", avail);
-
-  while (avail > MAX (512, visual->spf) * visual->bps) {
+  while (TRUE) {
     gboolean need_skip;
     const guint16 *data;
 
     GST_DEBUG_OBJECT (visual, "processing buffer");
 
+    avail = gst_adapter_available (visual->adapter);
+    GST_DEBUG_OBJECT (visual, "avail now %u", avail);
+
+    /* we need at least 512 samples */
+    if (avail < 512 * visual->bps)
+      break;
+
+    /* we need at least enough samples to make one frame */
+    if (avail < visual->spf * visual->bps)
+      break;
+
     if (visual->next_ts != -1) {
       gint64 qostime;
 
@@ -664,7 +675,11 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
           rate = VISUAL_AUDIO_SAMPLE_RATE_96000;
           break;
         default:
-          g_assert_not_reached ();
+          visual_object_unref (VISUAL_OBJECT (lbuf));
+          visual_object_unref (VISUAL_OBJECT (rbuf));
+          GST_ERROR_OBJECT (visual, "unsupported rate %d", visual->rate);
+          ret = GST_FLOW_ERROR;
+          goto beach;
           break;
       }
 
@@ -713,28 +728,27 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     ret = gst_pad_push (visual->srcpad, outbuf);
     outbuf = NULL;
 
-    GST_DEBUG_OBJECT (visual, "finished frame, flushing %u samples from input",
-        visual->spf);
   skip:
     /* interpollate next timestamp */
     if (visual->next_ts != -1)
       visual->next_ts += visual->duration;
 
-    /* Flush out the number of samples per frame * channels * sizeof (gint16) */
-    gst_adapter_flush (visual->adapter, MIN (avail, visual->spf * visual->bps));
+    GST_DEBUG_OBJECT (visual, "finished frame, flushing %u samples from input",
+        visual->spf);
+
+    /* Flush out the number of samples per frame */
+    gst_adapter_flush (visual->adapter, visual->spf * visual->bps);
 
     /* quit the loop if something was wrong */
     if (ret != GST_FLOW_OK)
       break;
-
-    /* see what we have left for next iteration */
-    avail = gst_adapter_available (visual->adapter);
   }
 
+beach:
+
   if (outbuf != NULL)
     gst_buffer_unref (outbuf);
 
-beach:
   gst_object_unref (visual);
 
   return ret;
@@ -861,6 +875,11 @@ plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (libvisual_debug, "libvisual", 0,
       "libvisual audio visualisations");
 
+#ifdef LIBVISUAL_PLUGINSBASEDIR
+  gst_plugin_add_dependency_simple (plugin, "HOME/.libvisual/actor",
+      LIBVISUAL_PLUGINSBASEDIR "/actor", NULL, GST_PLUGIN_DEPENDENCY_FLAG_NONE);
+#endif
+
   visual_log_set_verboseness (VISUAL_LOG_VERBOSENESS_LOW);
   visual_log_set_info_handler (libvisual_log_handler, (void *) GST_LEVEL_INFO);
   visual_log_set_warning_handler (libvisual_log_handler,
