@@ -287,6 +287,10 @@ gst_rtsp_connection_connect (GstRTSPConnection * conn, GTimeVal * timeout)
   else if (retval == -1)
     goto sys_error;
 
+  /* we can still have an error connecting on windows */
+  if (gst_poll_fd_has_error (conn->fdset, &conn->fd))
+    goto sys_error;
+
   gst_poll_fd_ignored (conn->fdset, &conn->fd);
 
 done:
@@ -506,6 +510,8 @@ gst_rtsp_connection_write (GstRTSPConnection * conn, const guint8 * data,
   gst_poll_set_controllable (conn->fdset, TRUE);
   gst_poll_fd_ctl_write (conn->fdset, &conn->fd, TRUE);
   gst_poll_fd_ctl_read (conn->fdset, &conn->fd, FALSE);
+  /* clear all previous poll results */
+  gst_poll_fd_ignored (conn->fdset, &conn->fd);
 
   to = timeout ? GST_TIMEVAL_TO_TIME (*timeout) : GST_CLOCK_TIME_NONE;
 
